#!/usr/local/bin/wish

# toxic.tcl
# see readme.txt


##############################
# UI routines
##############################

# proc ui

proc ui { } {

global DIR
set DIR [pwd]


# Set window title

wm title . {Toxic 0.2.0}

# Create and pack a main frame for the GUI

frame .mf
pack .mf

# Create menu buttons

button .mf.ttx_tox -relief raised -text {Convert TTX to TOX} -underline 0 -command ttxSelect
button .mf.ttx_detox -relief raised -text {Convert TOX to TTX} -underline 0 -command ttxToxSelect
button .mf.txml_tox -relief raised -text {Convert TXML to TOX} -underline 0 -command txmlSelect
button .mf.txml_detox -relief raised -text {Convert TOX to TXML} -underline 0 -command txmlToxSelect
label .mf.description1 -text {After converting back to TTX or TXML, remove the ".exp" extension from the TTX or TXML file. }
label .mf.description2 -text {For usage, refer to the readme file. }

# Pack (grid) menu buttons and labels

grid .mf.ttx_tox -column 0 -row 0 -sticky we
grid .mf.ttx_detox -column 1 -row 0 -sticky we
grid .mf.txml_tox -column 0 -row 1 -sticky we
grid .mf.txml_detox -column 1 -row 1 -sticky we
grid .mf.description1 -columnspan 2 -row 3 -sticky w
grid .mf.description2 -columnspan 2 -row 4 -sticky w

}


# ttxSelect
# file selection dialog for TTX file

proc ttxSelect {} {

global DIR

set fltyp {
{ "TTX file"	{ *.ttx }	} 
}

set Filename [tk_getOpenFile  -filetypes $fltyp -parent .]

catch { ttx_tox $Filename }

}


# ttxToxSelect
# file selection dialog for TOX file

proc ttxToxSelect {} {

global DIR

set fltyp {
{ "TTX TOX file"	{ *.ttx.tox }	} 
}

set Filename [tk_getOpenFile  -filetypes $fltyp -parent .]

catch { ttx_detox $Filename }

}

# txmlSelect
# file selection dialog for TTX file

proc txmlSelect {} {

global DIR

set fltyp {
{ "TXML file"	{ *.txml }	} 
}

set ttxFilename [tk_getOpenFile  -filetypes $fltyp -parent .]

catch { txml_tox $Filename }

}


# txmlToxSelect
# file selection dialog for TXML TOX file

proc txmlToxSelect {} {

global DIR

set fltyp {
{ "TXML TOX file"	{ *.txml.tox }	} 
}

set Filename [tk_getOpenFile  -filetypes $fltyp -parent .]

catch { txml_detox $Filename }

}


##############################
# ttx routines
#
##############################
# ttx_tox routine

proc ttx_tox {ttxFile} {

global srcLang trgLang
global tagno
global tagarray
global segno
global segarray

# initialize tag number

set tagno 1

# initialize target segment number

set segno 1

set langs [tlangident $ttxFile]

set srcLang [lindex $langs 0]

set trgLang [lindex $langs 1]

tfilehandler $ttxFile

}


proc tfilehandler {ttxFile} {

global tagno
global tagarray
global segno
global segarray
global srcLang trgLang
global tempFile

set openTTXComment {<!--leading_suppressed_ttx_body}
set closeTTXComment {trailing_suppressed_ttx_body-->}
set openToxSegComment {<!--leading_suppressed_toxtags}
set closeToxSegComment {trailing_suppressed_toxtags-->}
set patDeclr {<\?xml(.*?)>}


puts "LAUNCHING FILE HANDLER"

set toxExt {.tox}

set toxFile $ttxFile$toxExt


# create and open temporary file

set tempFile {temp.tox}

set tempFh [open $tempFile w]

fconfigure $tempFh -encoding unicode


# open ttx file for line-by-line reading

set ttxFh [open $ttxFile]

fconfigure $ttxFh -encoding unicode

gets $ttxFh 1stLine

if { [regexp $patDeclr $1stLine] == 1} {

puts $tempFh $1stLine

} else {

puts "no XML declaration found"

exit }

# write open toxic body tag

puts $tempFh {<toxic>}

# write leading XML comment

puts $tempFh $openTTXComment

# get each line of ttx file

while { [gets $ttxFh totalLine] >= 0 } {

# substitute xml comments

regsub -all -- {<!--} $totalLine {open_xml_comment} totalLine

regsub -all -- {-->} $totalLine {close_xml_comment} totalLine

regsub -all -- {&lt;!--} $totalLine {open_escaped_xml_comment} totalLine

regsub -all -- {--&gt;} $totalLine {close_escaped_xml_comment} totalLine

# substitute source paragraph tags 
# and problematic characters

set procdLine [tprocline $totalLine]

# write line to output file

puts $tempFh $procdLine

	}

# write closing XML comment for TTX body

puts $tempFh $closeTTXComment

# write opening XML comment for segments

puts $tempFh $openToxSegComment

# insert array of numbers and tags

foreach n [array names tagarray] { 

puts $tempFh <toxtag$n>$tagarray($n)</toxtag$n>

}

puts $tempFh $closeToxSegComment

# insert array of target segments

puts $tempFh {<toxictarget>}

foreach n [lsort -dictionary [array names segarray]] { 

puts $tempFh "<toxseg$n><target>$segarray($n)</target></toxseg$n>"

	}

puts $tempFh {</toxictarget>}

puts $tempFh {</toxic>}


close $ttxFh

close $tempFh

# open TOX file again and
# revert character substitution

trevertsub $toxFile

file delete temp.tox

}



# proc tlangident
# receives: name of TTX or TOX file
# outputs: source and target language codes

proc tlangident {tFile} {

global srcLang trgLang


puts "LAUNCHING TLANGIDENT"


# open ttx file and read entire content

set tFh [open $tFile]

fconfigure $tFh -encoding unicode

set tSize [file size $tFile]

set tCont [read $tFh $tSize]

# identify source and target language codes

set srcPat {SourceLanguage="(.+?)"}

regexp $srcPat $tCont match srcLang

set trgPat {TargetLanguage="(.+?)"}

regexp $trgPat $tCont match trgLang

set langs [list $srcLang $trgLang]

return $langs


# close ttx file again

close $tFh

}


# proc trevertsub

proc trevertsub {toxFile} {

global tempFile

# re-open temporary file, create and open tox file

set tempFh [open $tempFile r+]

fconfigure $tempFh -encoding unicode

set toxFh [open $toxFile w]

fconfigure $toxFh -encoding unicode


while { [gets $tempFh lineRevisited] >= 0 } {

# revert escaped angle brackets
# revert AND symbol

regsub -all {<andelteesc} $lineRevisited {\&lt;} lineRevisited

regsub -all {andgeeteesc>} $lineRevisited {\&gt;} lineRevisited

regsub -all AANNDD $lineRevisited {\&} lineRevisited

puts $toxFh $lineRevisited

}

close $tempFh

close $toxFh

}



# proc tprocline
# receives: unprocessed total line
# returns: prepared total line

proc tprocline {totalLine} {

global tagno
global segno
global tagarray
global segarray
global srcLang trgLang


# substitute escaped angle brackets
# substitute AND symbol

regsub -all {&lt;} $totalLine {<andelteesc} totalLine

regsub -all {&gt;} $totalLine {andgeeteesc>} totalLine

regsub -all {&} $totalLine AANNDD totalLine


# look for target segments in line

set trgPat1 {(.*?)(<Tuv Lang=")(}
set trgPat2 $trgLang
set trgPat3 {)(">)(.*?)(</Tuv>)(.*?)}

set patTarget $trgPat1$trgPat2$trgPat3

if { [regexp $patTarget $totalLine] == 0 } {

# if non found, return line verbatim

puts "PASSING LINE WITH UNCHANGED TUs"
return $totalLine

} else {

# if target segment found

while { [regexp $patTarget $totalLine] == 1 } {

# parse line for target segment

puts "PARSING TU"

regexp $patTarget $totalLine match sub1 sub2 sub3 sub4 sub5 sub6 sub7

# perform substitutions on target segment

puts "SUBSTITUTIONS ON TU"

set sub5 [ttagplaceholder $sub5]

set oSegPh {<toxseg}

set cSegPh {/>}

set segarray($segno) $sub5

regsub $patTarget $totalLine $sub1$oSegPh$segno$cSegPh$sub7 totalLine

# sub5 is target segment (within target tags), with unprocessed tags

incr segno

		}

puts "PASSING LINE WITH CHANGED TUs"

return $totalLine


	}

}



# proc ttagplaceholder
# receives: target segment (with full tags)
# outputs: target segment (with collapsed tags)

proc ttagplaceholder {sub5} {

global tagno
global tagarray
global srcLang trgLang

set tagPhPat1 {(<[^<>]+>)+}

set tagPhPat2 {xxtoxtag}

set tagPhPat3 yy

set tagPhPat4 {xxtoxtag([0-9]+)yy}

set tagPhPat5 {<toxtag\1/>}

while { [regexp $tagPhPat1 $sub5] == 1 } {

regexp $tagPhPat1 $sub5 tagseq

set tagarray($tagno) $tagseq

regsub $tagPhPat1 $sub5 $tagPhPat2$tagno$tagPhPat3 sub5

incr tagno

	}

regsub -all $tagPhPat4 $sub5 $tagPhPat5 sub5

return $sub5

}




##############################
# ttx_detox routine


proc ttx_detox {toxFile} {

# create segment and tag arrays

dtgetarrays $toxFile

set trgLang [dtlangident $toxFile]

dtpreproc $toxFile

# pass filename to dtfilehandler

dtfilehandler $toxFile

dtrevertchars

file delete temp1.tox

file delete temp2.tox

}



# proc dtpreproc
# recreates tox file without 
# XML comments and arrays

proc dtpreproc {toxFile} {

# open tox file for line-by-line reading

set striP1 {(ttx_body)|}
set striP2 {(ttx_body)|}
set striP3 {(leading_suppressed_toxtags)|}
set striP4 {(trailing_suppressed_toxtags)|}
set striP5 {(toxictarget>)|}
set striP6 {(</toxtag)|}
set striP7 {(</toxseg)|}
set striP8 {(<toxic>)|}
set striP9 {(</toxic>)|}
set striP10 {(<!DOCTYPE)}

# look for lines required only by the procedure

set striPat $striP1$striP2$striP3$striP4$striP5$striP6$striP7$striP8$striP9$striP10


set toxFh [open $toxFile]

fconfigure $toxFh -encoding unicode

# create and open temporary file 1

set temp1File {temp1.tox}

set temp1Fh [open $temp1File w]

fconfigure $temp1Fh -encoding unicode


while { [eof $toxFh] != 1 } { 

gets $toxFh newLine

puts {PREPROCEDTOX READING LINE BY LINE}

if { [regexp $striPat $newLine] == 1 } {

puts {DELETING THIS:}

puts $newLine

} else {

set procLine [dtharmlesschars $newLine]

puts $temp1Fh $procLine

puts {KEEPING THIS:}

puts $procLine

		}

	}

close $toxFh

close $temp1Fh

}




# proc dtlangident
# receives: name of TTX or TOX file
# outputs: source and target language codes

proc dtlangident {tFile} {

global srcLang trgLang


puts "LAUNCHING DTLANGIDENT"


# open ttx/tox file and read entire content

set tFh [open $tFile]

fconfigure $tFh -encoding unicode

set tSize [file size $tFile]

set tCont [read $tFh $tSize]

# identify source and target language codes

set srcPat {SourceLanguage="(.+?)"}

regexp $srcPat $tCont match srcLang

set trgPat {TargetLanguage="(.+?)"}

regexp $trgPat $tCont match trgLang

set langs [list $srcLang $trgLang]

# close ttx file again

close $tFh

puts "TARGET LANGUAGE IS $trgLang"

return $trgLang

}



proc dtfilehandler {toxFile} {

global temp2File
global ttxFile
global trgLang
global segarray
global tagarray

set LINENUMBER 0
set temp1File temp1.tox

puts "LAUNCHING FILE HANDLER"

set patOpenTTXComment {leading_suppressed_ttx_body}
set patCloseTTXComment {trailing_suppressed_ttx_body}

set segPhPat {<toxseg([0-9]+)/>}
set patDeclr {<\?xml(.*?)>}

set oTrgTag1 {<Tuv Lang="}
set oTrgTag2 {">}
set oTrgTag $oTrgTag1$trgLang$oTrgTag2
set cTrgTag {</Tuv>}

set ttxRoot [file rootname $toxFile]

set expSuffix {.exp}

set ttxFile $ttxRoot$expSuffix


# create and open temporary file 2

set temp2File {temp2.tox}

set temp2Fh [open $temp2File w]

fconfigure $temp2Fh -encoding unicode


# open temp1 file for line-by-line reading

set temp1Fh [open temp1.tox]

fconfigure $temp1Fh -encoding unicode

# read XML declaration

gets $temp1Fh 1stLine

if { [regexp $patDeclr $1stLine] == 1 } {

puts $temp2Fh $1stLine 

} else {

puts "no XML declaration found"

exit 
	}


# read file until eof

while { [gets $temp1Fh totalLine] != -1 } {

# look for segment placeholder in line

while { [regexp $segPhPat $totalLine] == 1 } {

puts "LOOKING FOR SEGMENT PLACEHOLDER"

# sub1 is segment number

regexp $segPhPat $totalLine match sub1

# using segment number, get target segment from array

set trgSeg $segarray($sub1)

puts {TARGET SEGMENT FOR THIS SEGMENT NUMBER IS:}
puts $trgSeg

# trgSeg is target segment with inline placeholders

# pass segment to tag processor
# get segment back with substituted tags

puts {ABOUT TO SUBSTITUTE INLINE TAGS}

# launch inline tag substitution routine

set insSeg [dtproctag $trgSeg]

# insSeg is target segment with verbose inline tags

puts {TARGET SEGMENT WITH SUBSTITUTED TAGS:}
puts $insSeg

# substitute escaped angle brackets
# substitute AND symbol

# insert target segment into line

regsub $segPhPat $totalLine $oTrgTag$insSeg$cTrgTag totalLine


puts {LINE WITH TARGET SEGMENt AND SUBSTITUTED TAGS}

# end of segment reading loop

		}

# write line to temporary file

puts {WRITING LINE TO TEMPORARY FILE}

puts $temp2Fh $totalLine

incr LINENUMBER

# end of line reading loop

	}

close $temp1Fh

close $temp2Fh

# end of procedure

}



# proc dtgetarrays
# receives: name of TOX file
# creates: segarray and tagarray

proc dtgetarrays {toxFile} {

global tagarray
global segarray
global trgLang

puts "LAUNCHING DTGETARRAYS"

# set toxic tag and toxic segment patterns

set tagPat {(<toxtag)([0-9]+)(>)(.*?)(</toxtag)([0-9]+)>}
set segPat {(<toxseg)([0-9]+)(><target>)(.*?)(</target>)(</toxseg)([0-9]+)(>)}
set endPat {</toxictarget>}

# open tox file for line-by-line reading

set toxFh [open $toxFile]

fconfigure $toxFh -encoding unicode

# read tox file line by line

while { [gets $toxFh totalLine] >= 0 } {

# look for toxic tags

if { [regexp $tagPat $totalLine] == 1 } {

regexp $tagPat $totalLine match sub1 sub2 sub3 sub4

# sub2 is the tag number
# sub4 is the tag content

set tagNo $sub2

# substitute characters with harmless characters

set tagCont [dtharmlesschars $sub4]

# add to array

set tagarray($tagNo) $tagCont

} elseif { [regexp $segPat $totalLine] == 1 } {

regexp $segPat $totalLine match sub1 sub2 sub3 sub4

# sub2 is the target segment number
# sub4 is the target segment content

set segNo $sub2

set segCont $sub4

# substitute characters with harmless characters

set segCont [dtharmlesschars $sub4]

# add to array

set segarray($segNo) $segCont

		}

	}

close $toxFh

}


# proc dtproctag
# receives: target segment (with collapsed tags)
# outputs: target segment (with full tags)

proc dtproctag {collapsedSeg} {

global tagarray

set tagPhPat {<toxtag([0-9]+)/>}

while { [regexp $tagPhPat $collapsedSeg] == 1 } {

regexp $tagPhPat $collapsedSeg match tagno

puts "PROCESSING TAG NUMBER $tagno"

# tagno is tag number

set inlineTagSeq $tagarray($tagno)

regsub $tagPhPat $collapsedSeg $inlineTagSeq collapsedSeg

	}

set expandedSeg $collapsedSeg

return $expandedSeg

}



# proc dtrevertchars
# revert substituted characters
# as revertsub, but for tox -> ttx

proc dtrevertchars {} {

global temp2File
global ttxFile

# re-open temporary file, create and open tox file

set temp2Fh [open $temp2File r+]

fconfigure $temp2Fh -encoding unicode

set ttxFh [open $ttxFile w]

fconfigure $ttxFh -encoding unicode


while { [gets $temp2Fh lineRevisited] >= 0 } {

# revert escaped angle brackets
# revert AND symbol

regsub -all {<andelteesc} $lineRevisited {\&lt;} lineRevisited

regsub -all {andgeeteesc>} $lineRevisited {\&gt;} lineRevisited

regsub -all AANNDD $lineRevisited {\&} lineRevisited

# revert substituted xml comments

regsub -all {open_xml_comment} $lineRevisited {<!--} lineRevisited

regsub -all {close_xml_comment} $lineRevisited {-->} lineRevisited

regsub -all -- {open_escaped_xml_comment} $lineRevisited {&lt;!--} lineRevisited

regsub -all -- {close_escaped_xml_comment} $lineRevisited {--&gt;} lineRevisited


puts $ttxFh $lineRevisited

	}

close $temp2Fh

close $ttxFh

}


proc dtharmlesschars {dangerString} {

# substitute characters temporarily

# substitute escaped angle brackets
# substitute AND symbol

regsub -all {&lt;} $dangerString {<andelteesc} harmlessString

regsub -all {&gt;} $dangerString {andgeeteesc>} harmlessString

regsub -all {&} $dangerString AANNDD harmlessString

return $harmlessString

}

##############################
# txml routines
#
#!/usr/local/bin/wish

# toxic_txml.tcl
# see readme.txt


##############################
# toxic txml 
# tox routine

proc txml_tox {txmlFile} {

# global srcLang trgLang
global tagno
global tagarray
global segno
global segarray

# initialize tag number

set tagno 1

# initialize target segment number

set segno 1

# set langs [tlangident $txmlFile]

# set srcLang [lindex $langs 0]

# set trgLang [lindex $langs 1]

txml_tfilehandler $txmlFile

}


# toxic txml
# proc txml_tfilehandler

proc txml_tfilehandler {txmlFile} {

global tagno
global tagarray
global segno
global segarray
# global srcLang trgLang
global tempFile

set openTXMLComment {<!--leading_suppressed_ttx_body}
set closeTXMLComment {trailing_suppressed_ttx_body-->}
set openToxSegComment {<!--leading_suppressed_toxtags}
set closeToxSegComment {trailing_suppressed_toxtags-->}
set patDeclr {<\?xml(.*?)>}


puts "LAUNCHING FILE HANDLER"

set toxExt {.tox}

set toxFile $txmlFile$toxExt


# create and open temporary file

set tempFile {temp.tox}

set tempFh [open $tempFile w]

# supports UTF-8 only, modify if necessary
# fconfigure $tempFh -encoding unicode


# open ttx file for line-by-line reading

set txmlFh [open $txmlFile]

# supports UTF-8 only, modify if necessary
# fconfigure $txmlFh -encoding unicode

gets $txmlFh 1stLine

if { [regexp $patDeclr $1stLine] == 1} {

puts $tempFh $1stLine

} else {

puts "no XML declaration found"

exit }

# write open toxic body tag

puts $tempFh {<toxic>}

# write leading XML comment

puts $tempFh $openTXMLComment

# get each line of ttx file

while { [gets $txmlFh totalLine] >= 0 } {

# create temporary source segments

regsub -all {<source>} $totalLine {<sourcetmp>} totalLine

# substitute xml comments

regsub -all -- {<!--} $totalLine {open_xml_comment} totalLine

regsub -all -- {-->} $totalLine {close_xml_comment} totalLine

regsub -all -- {&lt;!--} $totalLine {open_escaped_xml_comment} totalLine

regsub -all -- {--&gt;} $totalLine {close_escaped_xml_comment} totalLine

# substitute source paragraph tags 
# and problematic characters

set procdLine [txml_tprocline $totalLine]

# write line to output file

puts $tempFh $procdLine

	}

# write closing XML comment for TXML body

puts $tempFh $closeTXMLComment

# write opening XML comment for segments

puts $tempFh $openToxSegComment

# insert array of numbers and tags

foreach n [array names tagarray] { 

puts $tempFh <toxtag$n>$tagarray($n)</toxtag$n>

}

puts $tempFh $closeToxSegComment

# insert array of target segments

puts $tempFh {<toxictarget>}

foreach n [lsort -dictionary [array names segarray]] { 

puts $tempFh "<toxseg$n><target>$segarray($n)</target></toxseg$n>"

	}

puts $tempFh {</toxictarget>}

puts $tempFh {</toxic>}


close $txmlFh

close $tempFh

# open TOX file again and
# revert character substitution

txml_trevertsub $toxFile

file delete temp.tox

}



# proc tlangident
# receives: name of TTX or TOX file
# outputs: source and target language codes
# not required for txml

# proc tlangident {tFile} {

# global srcLang trgLang


# puts "LAUNCHING TLANGIDENT"


# open ttx file and read entire content

# set tFh [open $tFile]

# fconfigure $tFh -encoding unicode

# set tSize [file size $tFile]

# set tCont [read $tFh $tSize]

# identify source and target language codes

# set srcPat {SourceLanguage="(.+?)"}

# regexp $srcPat $tCont match srcLang

# set trgPat {TargetLanguage="(.+?)"}

# regexp $trgPat $tCont match trgLang

# set langs [list $srcLang $trgLang]

# return $langs


# close ttx file again

# close $tFh

# }


# proc txml_trevertsub

proc txml_trevertsub {toxFile} {

global tempFile

# re-open temporary file, create and open tox file

set tempFh [open $tempFile r+]

# supports UTF-8 only, modify if necessary
# fconfigure $tempFh -encoding unicode

set toxFh [open $toxFile w]

# supports UTF-8 only, modify if necessary
# fconfigure $toxFh -encoding unicode


while { [gets $tempFh lineRevisited] >= 0 } {

# revert escaped angle brackets
# revert AND symbol

regsub -all {<andelteesc} $lineRevisited {\&lt;} lineRevisited

regsub -all {andgeeteesc>} $lineRevisited {\&gt;} lineRevisited

regsub -all AANNDD $lineRevisited {\&} lineRevisited

puts $toxFh $lineRevisited

}

close $tempFh

close $toxFh

}



# proc txml_tprocline
# receives: unprocessed total line
# returns: prepared total line

proc txml_tprocline {totalLine} {

global tagno
global segno
global tagarray
global segarray


# substitute escaped angle brackets
# substitute AND symbol

regsub -all {&lt;} $totalLine {<andelteesc} totalLine

regsub -all {&gt;} $totalLine {andgeeteesc>} totalLine

regsub -all {&} $totalLine AANNDD totalLine


regsub -all {<source>} $totalLine {<sourcetmp} totalLine


# look for temp source segments in line

set srcPat1 {(.*?)}
set srcPat2 {(<sourcetmp>)}
set srcPat3 {(</source>)}

set oSrcTag {<source>}
set oTrgTag {<target>}
set cTrgTag {</target>}

set patSource $srcPat1$srcPat2$srcPat1$srcPat3$srcPat1

set patTarget $oTrgTag$srcPat1$cTrgTag$srcPat1

# set patTarget $oTrgTag$s3$cTrgTag

if { [regexp $patSource $totalLine] == 0 } {

# if non found, return line verbatim

puts "PASSING LINE WITH UNCHANGED TUs"
return $totalLine

} else {

# if source segment found

while { [regexp $patSource $totalLine] == 1 } {

# parse line for target segment

puts "PARSING TU"

regexp $patSource $totalLine match sub1 sub2 sub3 sub4 sub5

# create target segments

regsub $patSource $totalLine $sub1$oSrcTag$sub3$sub4$oTrgTag$sub3$cTrgTag$sub5 totalLine


# perform substitutions on target segment

puts "SUBSTITUTIONS ON TU"

set sub3 [txml_ttagplaceholder $sub3]

set oSegPh {<toxseg}

set cSegPh {/>}

set segarray($segno) $sub3


regexp $patTarget $totalLine match sub1 sub2 sub3 sub4

regsub $patTarget $totalLine $sub1$oSegPh$segno$cSegPh$sub5 totalLine


# sub3 is target segment (within target tags), with unprocessed tags

incr segno

		}

puts "PASSING LINE WITH CHANGED TUs"

return $totalLine


	}

}



# proc txml_ttagplaceholder
# receives: target segment (with full tags)
# outputs: target segment (with collapsed tags)

proc txml_ttagplaceholder {sub3} {

global tagno
global tagarray
# global srcLang trgLang

set tagPhPat1 {(<[^<>]+>)+}

set tagPhPat2 {xxtoxtag}

set tagPhPat3 yy

set tagPhPat4 {xxtoxtag([0-9]+)yy}

set tagPhPat5 {<toxtag\1/>}

while { [regexp $tagPhPat1 $sub3] == 1 } {

regexp $tagPhPat1 $sub3 tagseq

set tagarray($tagno) $tagseq

regsub $tagPhPat1 $sub3 $tagPhPat2$tagno$tagPhPat3 sub3

incr tagno

	}

regsub -all $tagPhPat4 $sub3 $tagPhPat5 sub3

return $sub3

}


##############################
# txml detox routine

proc txml_detox {toxFile} {

# create segment and tag arrays

txml_dtgetarrays $toxFile

txml_dtpreproc $toxFile

# pass filename to txml_dtfilehandler

txml_dtfilehandler $toxFile

txml_dtrevertchars

file delete temp1.tox

file delete temp2.tox

}



# proc txml_dtpreproc
# recreates tox file without 
# XML comments and arrays

proc txml_dtpreproc {toxFile} {

# open tox file for line-by-line reading

set striP1 {(ttx_body)|}
set striP2 {(ttx_body)|}
set striP3 {(leading_suppressed_toxtags)|}
set striP4 {(trailing_suppressed_toxtags)|}
set striP5 {(toxictarget>)|}
set striP6 {(</toxtag)|}
set striP7 {(</toxseg)|}
set striP8 {(<toxic>)|}
set striP9 {(</toxic>)|}
set striP10 {(<!DOCTYPE)}

# look for lines required only by the procedure

set striPat $striP1$striP2$striP3$striP4$striP5$striP6$striP7$striP8$striP9$striP10


set toxFh [open $toxFile]

# UTF-8 only, modify if necessary
# fconfigure $toxFh -encoding unicode

# create and open temporary file 1

set temp1File {temp1.tox}

set temp1Fh [open $temp1File w]

# UTF-8 only, modify if necessary
# fconfigure $temp1Fh -encoding unicode


while { [eof $toxFh] != 1 } { 

gets $toxFh newLine

puts {PREPROCEDTOX READING LINE BY LINE}

if { [regexp $striPat $newLine] == 1 } {

puts {DELETING THIS:}

puts $newLine

} else {

set procLine [txml_dtharmlesschars $newLine]

puts $temp1Fh $procLine

puts {KEEPING THIS:}

puts $procLine

		}

	}

close $toxFh

close $temp1Fh

}




# proc txml_dtlangident
# required?

# receives: name of TTX or TOX file
# outputs: source and target language codes

# proc dtlangident {tFile} {

# global srcLang trgLang


# puts "LAUNCHING DTLANGIDENT"


# open ttx/tox file and read entire content

# set tFh [open $tFile]

# fconfigure $tFh -encoding unicode

# set tSize [file size $tFile]

# set tCont [read $tFh $tSize]

# identify source and target language codes

# set srcPat {SourceLanguage="(.+?)"}

# regexp $srcPat $tCont match srcLang

# set trgPat {TargetLanguage="(.+?)"}

# regexp $trgPat $tCont match trgLang

# set langs [list $srcLang $trgLang]

# close ttx file again

# close $tFh

# puts "TARGET LANGUAGE IS $trgLang"

# return $trgLang

# }



# proc txml_dtfilehandler

proc txml_dtfilehandler {toxFile} {

global temp2File
global txmlFile
global segarray
global tagarray

set LINENUMBER 0
set temp1File temp1.tox

puts "LAUNCHING FILE HANDLER"

set patOpenTXMLComment {leading_suppressed_ttx_body}
set patCloseTXMLComment {trailing_suppressed_ttx_body}

set segPhPat {<toxseg([0-9]+)/>}
set patDeclr {<\?xml(.*?)>}

set oTrgTag {<target>}
set cTrgTag {</target>}


set txmlRoot [file rootname $toxFile]

set expSuffix {.exp}

set txmlFile $txmlRoot$expSuffix


# create and open temporary file 2

set temp2File {temp2.tox}

set temp2Fh [open $temp2File w]

# UTF-8 only, modify if necessary
# fconfigure $temp2Fh -encoding unicode

# open temp1 file for line-by-line reading

set temp1Fh [open temp1.tox]

# UTF-8 only, modify if necessary
# fconfigure $temp1Fh -encoding unicode

# read XML declaration

gets $temp1Fh 1stLine

if { [regexp $patDeclr $1stLine] == 1 } {

puts $temp2Fh $1stLine 

} else {

puts "no XML declaration found"

exit 
	}


# read file until eof

while { [gets $temp1Fh totalLine] != -1 } {

# look for segment placeholder in line

while { [regexp $segPhPat $totalLine] == 1 } {

puts "LOOKING FOR SEGMENT PLACEHOLDER"

# sub1 is segment number

regexp $segPhPat $totalLine match sub1

# using segment number, get target segment from array

set trgSeg $segarray($sub1)

puts {TARGET SEGMENT FOR THIS SEGMENT NUMBER IS:}
puts $trgSeg

# trgSeg is target segment with inline placeholders

# pass segment to tag processor
# get segment back with substituted tags

puts {ABOUT TO SUBSTITUTE INLINE TAGS}

# launch inline tag substitution routine

set insSeg [txml_dtproctag $trgSeg]

# insSeg is target segment with verbose inline tags

puts {TARGET SEGMENT WITH SUBSTITUTED TAGS:}
puts $insSeg

# substitute escaped angle brackets
# substitute AND symbol

# insert target segment into line

regsub $segPhPat $totalLine $oTrgTag$insSeg$cTrgTag totalLine


puts {LINE WITH TARGET SEGMENt AND SUBSTITUTED TAGS}

# end of segment reading loop

		}

# write line to temporary file

puts {WRITING LINE TO TEMPORARY FILE}

puts $temp2Fh $totalLine

incr LINENUMBER

# end of line reading loop

	}

close $temp1Fh

close $temp2Fh

# end of procedure

}



# proc txml_dtgetarrays
# receives: name of TOX file
# creates: segarray and tagarray

proc txml_dtgetarrays {toxFile} {

global tagarray
global segarray

puts "LAUNCHING DTGETARRAYS"

# set toxic tag and toxic segment patterns

set tagPat {(<toxtag)([0-9]+)(>)(.*?)(</toxtag)([0-9]+)>}
set segPat {(<toxseg)([0-9]+)(><target>)(.*?)(</target>)(</toxseg)([0-9]+)(>)}
set endPat {</toxictarget>}

# open tox file for line-by-line reading

set toxFh [open $toxFile]

# UTF-8 only, modify if necessary
# fconfigure $toxFh -encoding unicode

# read tox file line by line

while { [gets $toxFh totalLine] >= 0 } {

# look for toxic tags

if { [regexp $tagPat $totalLine] == 1 } {

regexp $tagPat $totalLine match sub1 sub2 sub3 sub4

# sub2 is the tag number
# sub4 is the tag content

set tagNo $sub2

# substitute characters with harmless characters

set tagCont [txml_dtharmlesschars $sub4]

# add to array

set tagarray($tagNo) $tagCont

} elseif { [regexp $segPat $totalLine] == 1 } {

regexp $segPat $totalLine match sub1 sub2 sub3 sub4

# sub2 is the target segment number
# sub4 is the target segment content

set segNo $sub2

set segCont $sub4

# substitute characters with harmless characters

set segCont [txml_dtharmlesschars $sub4]

# add to array

set segarray($segNo) $segCont

		}

	}

close $toxFh

}


# proc txml_dtproctag
# receives: target segment (with collapsed tags)
# outputs: target segment (with full tags)

proc txml_dtproctag {collapsedSeg} {

global tagarray

set tagPhPat {<toxtag([0-9]+)/>}

while { [regexp $tagPhPat $collapsedSeg] == 1 } {

regexp $tagPhPat $collapsedSeg match tagno

puts "PROCESSING TAG NUMBER $tagno"

# tagno is tag number

set inlineTagSeq $tagarray($tagno)

regsub $tagPhPat $collapsedSeg $inlineTagSeq collapsedSeg

	}

set expandedSeg $collapsedSeg

return $expandedSeg

}



# proc txml_dtrevertchars
# revert substituted characters
# as revertsub, but for tox -> txml

proc txml_dtrevertchars {} {

global temp2File
global txmlFile

# re-open temporary file, create and open tox file

set temp2Fh [open $temp2File r+]

# UTF-8 only; modify if necessary
# fconfigure $temp2Fh -encoding unicode

set txmlFh [open $txmlFile w]

# UTF-8 only; modify if necessary
# fconfigure $txmlFh -encoding unicode

while { [gets $temp2Fh lineRevisited] >= 0 } {

# revert escaped angle brackets
# revert AND symbol

regsub -all {<andelteesc} $lineRevisited {\&lt;} lineRevisited

regsub -all {andgeeteesc>} $lineRevisited {\&gt;} lineRevisited

regsub -all AANNDD $lineRevisited {\&} lineRevisited

# revert substituted xml comments

regsub -all {open_xml_comment} $lineRevisited {<!--} lineRevisited

regsub -all {close_xml_comment} $lineRevisited {-->} lineRevisited

regsub -all -- {open_escaped_xml_comment} $lineRevisited {&lt;!--} lineRevisited

regsub -all -- {close_escaped_xml_comment} $lineRevisited {--&gt;} lineRevisited


puts $txmlFh $lineRevisited

	}

close $temp2Fh

close $txmlFh

}


# proc txml_dtharmlesschars

proc txml_dtharmlesschars {dangerString} {

# substitute characters temporarily

# substitute escaped angle brackets
# substitute AND symbol

regsub -all {&lt;} $dangerString {<andelteesc} harmlessString

regsub -all {&gt;} $dangerString {andgeeteesc>} harmlessString

regsub -all {&} $dangerString AANNDD harmlessString

return $harmlessString

}


##############################

if { [llength $argv] == 0 } {

ui

} else {

set argCommand [lindex $argv 0]

set fileList [lrange $argv 1 end]
set i 0
foreach inputFile $fileList {
	$argCommand $inputFile
	incr i

}

exit

}

##############################

